﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Text.RegularExpressions;
using System.Diagnostics;
using Infragistics.Controls.Menus;
using Infragistics.Controls.Primitives;

namespace WpfApplication2
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(Window_Loaded);
        }
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.LoadData();
        }
        private void LoadData()
        {
            using (var sr = new StreamReader(@"..\..\Cameras.xml"))
            {
                var serializer = new XmlSerializer(typeof(List<Camera>));
                var cameras = (List<Camera>)serializer.Deserialize(sr);

                var tree = Stakeholder.GroupByVendor(cameras);
                
                this.cameraDataTree.ItemsSource = tree;


            }            
        }

        private void CameraDataTree_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            foreach (XamDataTreeNode n in ((sender as ExpansionIndicator).Tag as XamDataTreeNodeControl).Node.Nodes)
                n.IsExpanded = !n.IsExpanded;
        }
    
    }
    [DebuggerDisplay("{StakeholderName}")]
    public class Stakeholder
    {
        public static List<Stakeholder> GroupByVendor(List<Camera> cameras)
        {
            List<Stakeholder> tree = null;

            if (cameras != null && cameras.Count != 0)
            {
                var grouping = from c in cameras
                               orderby c.VendorRoot
                               group c by c.VendorRoot into g
                               let Vendor = g.First().Vendor
                               select new { Key = Vendor, Cameras = g };

                tree = new List<Stakeholder>();

                foreach (var group in grouping)
                {
                    tree.Add(new Stakeholder(group.Key, group.Cameras.ToList()));
                }
            }
            return tree;
        }
        private Stakeholder(string name, List<Camera> cameras)
        {
            string[] splitParts;
            if (!string.IsNullOrWhiteSpace(name) && (Camera.VendorSplitter.IsMatch(name) && (splitParts = Camera.VendorSplitter.Split(name)).Length > 1))
            {
                var stakeholderName = splitParts[0];
                this.StakeholderName = stakeholderName.Trim();

                if (cameras.Exists(x => x.Vendor.Equals(stakeholderName)))
                {
                    this.Cameras = cameras.Where(x => x.Vendor.Equals(stakeholderName)).ToList();
                }
                else
                {
                    cameras.ForEach(x => x.Vendor = TrimVendorParts(x));
                    var grouping = from c in cameras
                                   orderby c.VendorRoot
                                   group c by c.VendorRoot into g
                                   let Vendor = g.First().Vendor
                                   select new { Key = Vendor, Cameras = g };

                    if (grouping.Count() != 0)
                    {
                        this.Subgroups = new List<Stakeholder>();

                        foreach (var group in grouping)
                        {
                            this.Subgroups.Add(new Stakeholder(group.Key, group.Cameras.ToList()));
                        }
                    }
                }
            }
            else
            {
                this.StakeholderName = name.Trim();
                this.Cameras = cameras;
            }
        }
        public string StakeholderName { get; set; } 
        public List<Camera> Cameras { get; set; }
        public List<Stakeholder> Subgroups { get; set; }

        private string TrimVendorParts(Camera camera)
        {
            if (camera.VendorParts.Count > 1)
            {
                camera.VendorParts.RemoveAt(0);
                return string.Join(" > ", camera.VendorParts).Trim();
            }
            else
            {
                return camera.VendorParts[0];
            }
        }
    }
    [DebuggerDisplay("{CameraName}")]
    public class Camera
    {
        private List<string> vendorParts;
        public static readonly Regex VendorSplitter = new Regex(@"\s\>\s", RegexOptions.Compiled | RegexOptions.Singleline); 

        public int CameraId { get; set; }
        public string CameraName { get; set; }
        public string Vendor { get; set; }

        public string VendorRoot
        {
            get { return VendorParts[0]; }
        }
        public List<string> VendorParts
        {
            get
            {
                if (vendorParts == null && Vendor != null)
                {
                    vendorParts = new List<string>(VendorSplitter.Split(Vendor));
                }
                return vendorParts;
            }
        }
    }
}
